from django.contrib import admin
from .models import *
from django.contrib.auth.models import Group
from django.http import HttpResponseRedirect
from django.urls import reverse



class SliderAdmin(admin.ModelAdmin):
	list_display = ('name',)


class CustomerAdmin(admin.ModelAdmin):
	list_display = ('user', 'name', 'email', )
	search_fields = ('name', 'email', )
	list_filter = ('email',)


class ProductCategoryAdmin(admin.ModelAdmin):
	list_display = ('name', )
	search_fields = ('name', )
	list_filter = ['name']

class ClassificationAdmin(admin.ModelAdmin):
	list_display = ('name', )
	search_fields = ('name', )
	list_filter = ['name']

class ProductDetailsInline(admin.TabularInline):
    model = productDetails
    extra = 1  # Number of empty forms to display


class ProductImageInline(admin.TabularInline):
    model = ProductImage
    extra = 1 


class ProductAdmin(admin.ModelAdmin):
    list_display = ('name', 'price', 'digital', 'product_slug')
    search_fields = ('name', )
    prepopulated_fields = {'product_slug': ('name', )}
    list_filter = ['digital',]
    inlines = [ProductDetailsInline, ProductImageInline]

class ProductImageAdmin(admin.ModelAdmin):
	# list_display = ('name', )
	# list_filter = ('name', )
	search_fields = ('name', )

class OrderAdmin(admin.ModelAdmin):
    list_display = ('get_customer', 'get_customer_email', 'get_phone_number', 'get_ordered_products', 'get_shipping_info', 'date_ordered', 'complete')
    search_fields = ('transaction_id', )
    list_per_page = 10
    list_filter = ['date_ordered', 'customer', 'complete', 'transaction_id', ]

    def get_customer(self, obj):
        if obj.customer and obj.customer.user:
            return obj.customer.user.username
        elif obj.customer:
              return obj.customer.name
        return None
    get_customer.short_description = 'Customer'

    def get_customer_email(self, obj):
        if obj.customer and obj.customer.user:
            return obj.customer.user.email
        elif obj.customer:
              return obj.customer.email
        return None
    get_customer_email.short_description = 'Customer Email'

    def get_ordered_products(self, obj):
        return ", ".join([item.product.name for item in obj.orderitem_set.all()])
    get_ordered_products.short_description = 'Ordered Products'

    def get_shipping_info(self, obj):
        shipping_address = ShippingAddress.objects.filter(order=obj).first()
        if shipping_address:
            return f"{shipping_address.country}, {shipping_address.state}, {shipping_address.city}, {shipping_address.address}"
        return None
    get_shipping_info.short_description = 'Shipping Info'

    def get_phone_number(self, obj):
        if obj.customer:
            shipping_address = ShippingAddress.objects.filter(order=obj).first()
            if shipping_address:
                return shipping_address.zipcode
        return None
    get_phone_number.short_description = 'Phone Number'


class OrderItemAdmin(admin.ModelAdmin):
    list_display = ('product',  'quantity', 'date_added')



class ShippingAddressAdmin(admin.ModelAdmin):
	list_display = ('address', 'city', 'state', 'country', 'zipcode', 'date_added')
	search_fields = ('zipcode', 'city', 'state', 'country', )
	list_per_page = 10
	list_filter = ['country', 'state', 'city', 'zipcode', ]


class DetailsAdmin(admin.ModelAdmin):
	list_display = ('name', 'brand', 'size')
	search_fields = ('brand', 'size', )
	list_per_page = 10
	list_filter = ['brand', 'size', ]


admin.site.register(productDetails, DetailsAdmin)

admin.site.register(Customer, CustomerAdmin)
admin.site.register(Product, ProductAdmin)
admin.site.register(Order, OrderAdmin)
admin.site.register(OrderItem, OrderItemAdmin)
admin.site.register(ShippingAddress, ShippingAddressAdmin)
admin.site.register(ProductCategory, ProductCategoryAdmin)
admin.site.register(Slider, SliderAdmin)
admin.site.register(ProductImage, ProductImageAdmin)
admin.site.register(Classification, ClassificationAdmin)


admin.site.site_header = "Virgin Admin Dashboard"
admin.site.site_footer = "Virgin Admin Dashboard"
admin.site.site_title = "Virgin Admin Page"
admin.site.index_title = " Wellcome to Virgin Fashion"
# admin.site.site_url = "virgin "
