from django.shortcuts import render
from django.http import JsonResponse
import json
import datetime
from .models import *
from .utils import cookieCart, cartData, guestOrder
from django.views.generic import ListView, DetailView
from django.db.models import Count
from django.shortcuts import render, get_object_or_404



def product_list(request, category_slug=None, classification_slug=None):
    category = None
    classification = None
    products = Product.objects.all()

    if category_slug:
        category = get_object_or_404(ProductCategory, slug=category_slug)
        products = products.filter(category=category)

    if classification_slug:
        classification = get_object_or_404(Classification, slug=classification_slug)
        products = products.filter(classification=classification)

    context = {
        'category': category,
        'classification': classification,
        'products': products,
    }
    return render(request, 'store/product_list.html', context)



def Home(request):
    data = cartData(request)

    cartItems = data['cartItems']
    order = data['order']
    items = data['items']

    sliders = Slider.objects.all()


    # Get the most sold product
    # most_sold_product = OrderItem.objects.values('product').annotate(total=Count('id')).order_by('-total')[:6]
    most_sold_products = OrderItem.objects.values('product_id').annotate(total=Count('id')).order_by('-total')[:6]
    most_sold_products_ids = [product['product_id'] for product in most_sold_products]
    most_sold_products_info = Product.objects.filter(id__in=most_sold_products_ids)

    # get new added products
    products = Product.objects.order_by("-created_at")[:6]

    # get list of categories
    categories = ProductCategory.objects.all()

    # today's deals
    deals = Product.objects.filter(id__in=most_sold_products_ids).order_by("-created_at")[:2]

    # classifications
    classification = Classification.objects.all()

    context = {'categories': categories, 'products':products, 
               'cartItems':cartItems, 'sliders': sliders, 'populars': most_sold_products_info,
                'deals': deals, 'classification': classification}

    return render(request, 'store/main.html', context)


def Store(request):
    # get new added products
    products = Product.objects.order_by("-created_at")
    # get list of categories
    categories = ProductCategory.objects.all()
    
    context = {'categories': categories, 'products':products}

    return render(request, 'store/store.html', context)



def product_detail(request, product_slug):
    product = get_object_or_404(Product, product_slug=product_slug)
    product_details = productDetails.objects.filter(name=product).first()

    context = {
        'product': product,
        'product_details': product_details,
    }

    return render(request, 'store/productdetails.html', context)


def Cart(request):
    data = cartData(request)

    cartItems = data['cartItems']
    order = data['order']
    items = data['items']

    context = {'items':items, 'order':order, 'cartItems':cartItems}
    return render(request, 'store/cart.html', context)

def cartItems(request):
    data = cartData(request)
    CartItems = data['cartItems']
    return {
        "cartItems": CartItems
    }

def Checkout(request):
    data = cartData(request)

    cartItems = data['cartItems']
    order = data['order']
    items = data['items']

    context = {'items':items, 'order':order, 'cartItems':cartItems}
    return render(request, 'store/checkout.html', context)

def updateItem(request):
    data = json.loads(request.body)
    productId = data['productId']
    action = data['action']
    # print('Action:', action)
    # print('Product:', productId)

    customer = request.user.customer
    product = Product.objects.get(id=productId)
    order, created = Order.objects.get_or_create(customer=customer, complete=False)

    orderItem, created = OrderItem.objects.get_or_create(order=order, product=product)

    if action == 'add':
        orderItem.quantity = (orderItem.quantity + 1)
    elif action == 'remove':
        orderItem.quantity = (orderItem.quantity - 1)

    orderItem.save()

    if orderItem.quantity <= 0:
        orderItem.delete()

    return JsonResponse('Item was added', safe=False)


def processOrder(request):
    transaction_id = datetime.datetime.now().timestamp()
    data = json.loads(request.body)

    if request.user.is_authenticated:
        customer = request.user.customer
        order, created = Order.objects.get_or_create(customer=customer, complete=False)
        order.complete = True  # Set the order to complete

        # Check if shipping is required
        if order.shipping and not ShippingAddress.objects.filter(order=order).exists():
            ShippingAddress.objects.create(
                customer=customer,
                order=order,
                address=data['shipping']['address'],
                city=data['shipping']['city'],
                state=data['shipping']['state'],
                country=data['shipping']['city'],
                zipcode=data['shipping']['zipcode'],
                info=data['shipping'].get('info', ''),
            )
    else:
        customer, order = guestOrder(request, data)

    total = float(data['form']['total'])
    order.transaction_id = transaction_id
    total = format(total, '.2f')
    order_total = format(order.get_cart_total, '.2f')

    print(total)
    print(order.get_cart_total)
    print(f"Order complete before: {order.complete}")

    # Update the order completion status
    if total == order_total:
        order.complete = True  # Set the order to complete
    else:
        order.complete = False  # Ensure the order is not marked complete if totals do not match

        # Set attribute to avoid triggering the signal
    request.user._order_save = True
    order.save()  # Save the order state
    del request.user._order_save
    print(f"Order complete after: {order.complete}")

    return JsonResponse('Payment Completed', safe=False)

def SearchView(request):
    if request.method == 'POST':
        searching = request.POST.get('search')
        print(searching)
        results = Product.objects.filter(name__contains=searching)
        context = {
            'search_results':results
        }
        return render(request, 'store/search_result.html', context)


def single_slug(request, single_slug):
    product_urls = {}
    products = [c.product_slug for c in Product.objects.all()]
    if single_slug in products:
        matching_details = productDetails.objects.filter(name__product_slug=single_slug)
        for m in matching_details.all():
        	products = matching_details
        	product_urls[m] = products.product_slug

    return render(request, "store/productdetails.html", {"part_ones": product_urls})

    return HttpResponse(f"{single_slug} No Details Found")


def about(request):
    return render(request, 'store/about.html')
